import * as mysql from 'mysql'

import { MySqlConfiguration } from "./libs/config"
import MySqlConnection from "./libs/connection"
import MySqlError from './libs/error'

export default class MySQL {

    private config: MySqlConfiguration

    constructor(config: MySqlConfiguration) {
        this.config = config
    }

    public open(): Promise<MySqlConnection> {

        return new Promise((resolve, reject) => {

            this.checkConfiguration().then(config => {

                let conn = mysql.createConnection(config)

                conn.connect(err => {
                    
                    if (err) {
                        let error = new MySqlError('Error connecting to host', err)
                        return reject(error)
                    }
    
                    resolve(conn)
                })
    
            }, (err) => {
                reject(err)
            })

        })

    }

    public query(query: string, values?: Array<any>): Promise<any> {
        
        return new Promise((resolve, reject) => {
            
            this.open().then(conn => {

                conn.query(query, values, (err, results) => {
                    
                    conn.end()
                    if (err) { 
                        let error = new MySqlError('Error while executing query', err)
                        return reject(error) 
                    }

                    resolve(results)
                })

            }, err => {
                reject(err)
            })
        })
    }
    
    private checkConfiguration(): Promise<MySqlConfiguration> {

        return new Promise((resolve, reject) => {

            if (!this.config || !this.config.host) {

                let error = new MySqlError('No configuration provided')
                return reject(error)
            } else {
                resolve(this.config)
            }
            
        })
    }

}